<?php
require_once(dirname(__FILE__).'/constants.php');
require_once(dirname(__FILE__).'/constants_changelog.php');
require_once(dirname(__FILE__).'/session.php');

class MYDB {
	public $connection;	// подключение
	public $db_ok;		// признак наличия подключения

	function __construct() {
		global $logger;
		$engine = DB_ENGINE;
		$host = DB_HOST;
		$db = DB_HOST.':'.DB_NAME;
		$charset = DB_CHARSET;
		$user = DB_USER;
		$pass = DB_PASS;

		$dsn = "$engine:host=$host;dbname=$db;charset=$charset";
		$opt = array(
			PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_BOTH,  // номера и имена колонок
			PDO::ATTR_CASE => PDO::CASE_LOWER, // переводить имена колонок в нижний регистр
			PDO::ATTR_ERRMODE => PDO::ERRMODE_SILENT // pdo предоставляет код ошибки
		);

		try {
			$this->connection = new PDO($dsn, $user, $pass, $opt);
			$this->db_ok = true;
			//$logger->log("database: connected");
		} catch (PDOException $e) {
			$this->db_ok = false;
			//echo 'database connection failed: ' . $e->getMessage();
			//$logger->log("database: connection failed: " . $e->getMessage());
		}
	}

	/* query -wrapper */
	function query($query) {
		return $this->connection->query($query);
	}

	/* prepare - wrapper */
	function prepare($query) {
		return $this->connection->prepare($query);
	}
	/* execute - wrapper with result array returning */
	function execute_res($q, $data) {
		$q->execute($data);
		return $q->fetchAll();
	}
	/* execute - wrapper without array returning */
	function execute($q, $data) {
		$q->execute($data);
	}
	
	// подтверждение пароля и пользователя
	function confirmUserPass($txt_login, $txt_password = null) {
		$q = <<<EOS
		select first 1 user_id from tb_user where user_name = :LOGIN and user_pass = :PASSWORD;
EOS;
		$q=$this->prepare($q);
		$q->bindValue(":LOGIN",$txt_login);
		$q->bindValue(":PASSWORD",$txt_password);
		$q->execute();
		$res = $q->fetchAll();
		
		if( !$res || empty($res[0]['user_id']) ) $res[0]['user_id'] = -1;
		return $res[0]['user_id'];	
	}


	// регистрация сессии (если сессия зарегистрирована, то обновляется время входа)
	function registerLoginSession($of_id,$user_id,$session_id,$start_time) {
		global $session, $logger;
	
		//$of_id = (int)$session->of_id;
		//$user_id = (int)$session->user_id;
		//$session_id = $session->session_id;
		//$start_time = (int)$session->start_time;
		
		$q = <<<EOS
		execute procedure of_web_registerloginsession(:USERID, :OFID, :USERWEBID, :STARTTIME)
EOS;

		$q=$this->prepare($q);
		$q->bindValue(":USERID",$user_id);
		$q->bindValue(":OFID",$of_id);
		$q->bindValue(":USERWEBID",$session_id);
		$q->bindValue(":STARTTIME",$start_time);
		
		$q->execute();
		$res = $q->fetchAll();
		
		return true;
	}

	
	// параметры пользователя
	function getUserInfo($txt_login, $txt_password = null) {
		global $logger;

		$q = <<<EOS
		select * from fm_get_userinfo(:LOGIN,:PASSWORD);
EOS;
		$q=$this->prepare($q);
		$q->bindValue(":LOGIN",$txt_login);
		$q->bindValue(":PASSWORD",$txt_password);
		$q->execute();
		$res = $q->fetchAll();
		
		foreach($res as $key => $row){
			$tabrow[$key] = array(
				'of_id' => $row['of_id'],
				'user_id' => $row['user_id'],
				'user_role' => $row['user_role'],
				'user_fio' => $row['user_fio']
			);
		}
		return $tabrow;
	}
	
	
	// список сотрудников фирмы
/*	
	function proc_get_emploeer($ofid) {
	global $logger;

		//$q = 'select * from tb_office_users';
		//$res = $this->connection->query($q)->fetchAll();
		$q = <<<EOS
		select 
			v1.user_id,
			v1.user_fio,
			v1.user_name,
			v1.user_photo_name,
			v1.user_photo_file,
			v2.group_name,
			v4.internal_number
		from 
			tb_office_users v1 left join tb_office_group v2 on v1.group_id = v2.group_id 
			
			left join tb_dials_list v3 on v1.user_id = v3.user_id
			left join tb_int_numbers v4 on v4.dials = v3.dials_id
			
		where v1.of_id=?;
EOS;
		$q = $this->prepare($q);
		$res = $this->execute_res($q,array($ofid));

		if ($q->errorCode() != '00000') {
			$logger->log("error: proc_get_emploeer", PEAR_LOG_ERR);
			$res = null;
		}
		
		foreach($res as $key => $row){
			$param = "'".implode("|",$row)."'";
			$tabrow[$key] = array(
				'user_id' => $row['user_id'],
				'user_fio' => $row['user_fio'],
				'user_name' => $row['user_name'],
				'user_photo_name' => $row['user_photo_name'],
				'user_photo_file' => $row['user_photo_file'],
				'group_name' => $row['group_name'],
				'internal_number' => $row['internal_number'],
				'param' => $param
			);
		}
		
		return $tabrow;
	}
*/
	function proc_get_emploeer($ofid) {
	global $logger;

		//$q = 'select * from tb_office_users';
		//$res = $this->connection->query($q)->fetchAll();
		$q = <<<EOS
		select 
			user_id,
			user_fio,
			user_name,
			user_photo_name,
			user_photo_file,
			group_name,
			internal_number
		from 
			FM_EMPLOYEE_LIST(?);
EOS;
		$q = $this->prepare($q);
		$res = $this->execute_res($q,array($ofid));

		if ($q->errorCode() != '00000') {
			$logger->log("error: proc_get_emploeer", PEAR_LOG_ERR);
			$res = null;
		}
		
		foreach($res as $key => $row){
			$param = "'".implode("|",$row)."'";
			$tabrow[$key] = array(
				'user_id' => $row['user_id'],
				'user_fio' => $row['user_fio'],
				'user_name' => $row['user_name'],
				'user_photo_name' => $row['user_photo_name'],
				'user_photo_file' => $row['user_photo_file'],
				'group_name' => $row['group_name'],
				'internal_number' => $row['internal_number'],
				'param' => $param
			);
		}
		
		return $tabrow;
	}














	// вытаскивает аватарки сотрудников из базы, создает файлы в ./content/img/tmp/ для загрузки на страничку
	function proc_create_emploeer_photo($ofid) {
	global $logger;

		$q = <<<EOS
		select 
			user_photo_name, 
			user_photo_file 
		from 
			tb_office_users 
		where of_id=?;
EOS;
		$q = $this->prepare($q);
		$res = $this->execute_res($q,array($ofid));
		
		foreach($res as $key => $row){
			
			$fname = !empty($row['user_photo_name']) ? './'.AVATAR_PATH.$row['user_photo_name'] : '';
			
			//echo $fname.'<br>';
			
			$fcontent = $row['user_photo_file'];
			$logger->log("fname = ".$fname, PEAR_LOG_DEBUG);
			if(!empty($fname)){
				$f = fopen($fname,"wb"); 
				fwrite($f,$fcontent);
				fclose($f);
			}
		}
		
	}
	
	// удаляет старые временные фалы из ./content/img/tmp/
	function proc_delete_emploeer_oldtmp_photo() {
		
		$timer = 7200;
		
		$dir_path = './'.AVATAR_PATH;
		if ($dh = opendir($dir_path)) {
			$time_real = time();
			while (($file = readdir($dh)) !== false) { 
				$time_file = filemtime($dir_path.$file);
				if ( $time_real-$time_file > $timer ) {
					if($file!='readme.txt' and $file!='..' ) unlink($dir_path.$file);
				}
			}		
			closedir($dh);
		}	
	}














	
	// информация о сотруднике
	function proc_get_emploeer_info($user_id) {
	global $logger;

		$q = <<<EOS
		select 
			user_id,
			user_fio,
			user_name,
			user_photo_name 
		from 
			tb_office_users 
		where user_id=?;
EOS;
		$q = $this->prepare($q);
		$res = $this->execute_res($q,array($user_id));

		if ($q->errorCode() != '00000') {
			$logger->log("error: proc_get_emploeer_info", PDO_LOG_ERR);
			$res = null;
		}
		
/*		foreach($res as $key => $row){
			$param = "'".implode("|",$row)."'";
			$tabrow[$key] = array(
				'user_id' => $row['user_id'],
				'user_fio' => $row['user_fio'],
				'user_name' => $row['user_name'],
				'user_photo_name' => $row['user_photo_name'],
				'param' => $param
			);
		}*/
			$tabrow = array(
				'user_id' => $res[0]['user_id'],
				'user_fio' => $res[0]['user_fio'],
				'user_name' => $res[0]['user_name'],
				'user_photo_name' => $res[0]['user_photo_name']
			);
		
		
		return $tabrow;
	}	


	// список групп сотрудников
	function proc_get_emploee_group($ofid) {
	global $logger;

		$q = <<<EOS
		select 
			group_id,
			group_name
		from 
			tb_office_group 
		where of_id=?;
EOS;
		$q = $this->prepare($q);
		$res = $this->execute_res($q,array($ofid));

		foreach($res as $key => $row){
			$param = "'".implode("|",$row)."'";
			$tabrow[$key] = array(
				'group_id' => $row['group_id'],
				'group_name' => $row['group_name'],
				'param' => $param
			);
		}
		
		return $tabrow;
	}	

	// сотрудники группы 
	function proc_get_group_employees($group_id) {

/*		$q = <<<EOS
		select 
			user_id,
			user_fio,
			user_photo_name,
			user_photo_file
		from 
			tb_office_users 
		where group_id=?;
EOS;*/
		$q = <<<EOS
		select 
			v1.user_id,
			v1.user_fio,
			v1.user_photo_name,
			v1.user_photo_file
		from 
			tb_office_users v1 inner join tb_office_user_group v2 on v1.user_id = v2.user_id
		where v2.group_id=?;
EOS;

		$q = $this->prepare($q);
		$res = $this->execute_res($q,array($group_id));

		foreach($res as $key => $row){
			$tabrow[$key] = array(
				'user_id' => $row['user_id'],
				'user_fio' => $row['user_fio'],
				'user_photo_name' => AVATAR_PATH.$row['user_photo_name']
			);

			$fname = !empty($row['user_photo_name']) ? AVATAR_PATH.$row['user_photo_name'] : '';
			$fcontent = $row['user_photo_file'];
			if(!empty($fname)){
				$f = fopen($fname,"wb"); 
				fwrite($f,$fcontent);
				fclose($f);
			}
		}
		
		return $tabrow;
	}	


	// группы сотрудников ---------------------------------------------------------------------------
	function proc_office_group(){
		//global $database, $logger;
	
		$ofid = 8; //$_REQUEST['ofid'];
	
		$q = <<<EOS
		select
		group_id,
		group_name
		from tb_office_group
		where of_id = ?;
EOS;
		$q = $this->prepare($q);
		$res = $this->execute_res($q,array($ofid));

		
		foreach($res as $key => $row){
			$param = "'".implode("|",$row)."'";
			$tabrow[$key] = array(
				'group_id' => $row['group_id'],
				'group_name' => $row['group_name']
			);
		}
		return $tabrow;
	}


	/**
	 * logOfficeChange - Logs change event to database.
	 * Returns false or code from database :)
	 * @param $log_changetype
	 * @param $log_office_id
	 * @param $log_userdata
	 * @param $log_msg
	 * @return bool
	 */
	function logOfficeChange($log_changetype, $log_office_id, $log_userdata, $log_msg) {
		global $session, $logger;

		//changelog
		$log_office_id_in = $log_office_id ? $log_office_id : 'null';
		$log_office_user_id = $session->logged_in ? $session->user_id : -1;
//		$log_msg .= $session->logged_in ? " Пользователь '{$session->username}' (id:{$session->userid})." : "";
//		$log_msg .= $session->logged_in ? " Пользователь '{$session->username}' (id:{$session->userid})(ip:{$session->user_ip})." : "";
		
		/*
		if ($session->checkUserRight('changelog_ip','A')) {
		$log_msg .= $session->logged_in ? " Пользователь '{$session->username}' (id:{$session->userid})(ip:{$session->user_ip})." : "";
		}
		else {
		$log_msg .= $session->logged_in ? " Пользователь '{$session->username}' (id:{$session->userid})." : "";
		}
		*/
/*
		$q = "select * from office_changelog_log($log_changetype, $log_office_id_in, $log_office_user_id, $log_userdata, " .
			"'".$this->connection->escape($log_msg)."')";
		$result = $this->connection->query($q);
		if (PEAR::isError($result)) {
			$logger->log("$q:". getPEARErrorInfo($result), PEAR_LOG_ERR);
			return false;
		}

		if ((!$dbarray = $result->fetchRow()) || !isset($dbarray['out_res'])) {
			return false;
		}

		return $dbarray['out_res'];
*/		
		$q = <<<EOS
		execute procedure office_changelog_log(:CHANGETYPE, :OFID, :USERID, :USERDATA, :MSG)
EOS;
		//$logger->log("$q     ".$user_id.'    '.$of_id.'    '.$session_id.'    '.$start_time, PEAR_LOG_DEBUG);

		$q=$this->prepare($q);
		$q->bindValue(":CHANGETYPE",$log_changetype);
		$q->bindValue(":OFID",$log_office_id_in);
		$q->bindValue(":USERID",$log_office_user_id);
		$q->bindValue(":USERDATA",$log_userdata);
		$q->bindValue(":MSG",$log_msg);
		
		$q->execute();
		$res = $q->fetchAll();
		
		return true;
	}


	// список графиков офиса
	function proc_get_emploee_graphic($ofid) {
	global $logger;

		$q = <<<EOS
		select
		v1.graph_name,
		substring(right('0000'||cast(v2.start_time as varchar(10)),4) from 1 for 2)||':'||
		substring(right('0000'||cast(v2.start_time as varchar(10)),4) from 3 for 2)
		as start_time,

		substring(right('0000'||cast(v2.end_time as varchar(10)),4) from 1 for 2)||':'||
		substring(right('0000'||cast(v2.end_time as varchar(10)),4) from 3 for 2)
		as end_time,

		v1.graph_id
		from tb_graph_list v1 inner join tb_graphics v2 on v1.graph_id = v2.graph_id
		where of_id = ?;
EOS;
		$q = $this->prepare($q);
		$res = $this->execute_res($q,array($ofid));

		foreach($res as $key => $row){
			$param = "'".implode("|",$row)."'";
			$tabrow[$key] = array(
				'graph_name' => $row['graph_name'],
				'start_time' => $row['start_time'],
				'end_time' => $row['end_time'],
				'graph_id' => $row['graph_id'],
				'param' => $param
			);
		}
		
		return $tabrow;
	}	


	
	// статистика для выгрузки (выборка берется из proc_get_stats_table в process.php, только без разбития по страницам)
	function get_stats($txt_date1,$txt_date2,$employee_id,$txt_calltype,$txt_ats_number,$txt_abonent_number) {
	global $logger,$session;
		$ofid = (int)$session->of_id;
		$employee_id = (!empty($employee_id)) ? 'and user_id in ('.$employee_id.')' : '';
		$txt_calltype = (!empty($txt_calltype)) ? 'and calltype in ('.$txt_calltype.')' : 'and 1<>1';
		$txt_abonent_number = trim($txt_abonent_number);
		$txt_abonent_number = (!empty($txt_abonent_number)) ? "and (numbera like '".$txt_abonent_number."' or numberb like '".$txt_abonent_number."')" : '';
		$txt_ats_number = (!empty($txt_ats_number)) ? 'and numbersystem = '.$txt_ats_number : '';
		$q = <<<EOS
		select 
		sessionid,
		creattime,
		numbera,
		numberb,
		realvolume,
		geographyname,
		distancename,
		pay,
		recordid,
		termreason,
		cause,
		answerdelay,
		flagsout,
		calltype,
		payablevolume,
		serviceblockid,
		messagetype,
		floatcash1,
		floatcash2,
		floatgraph,
		calluri,
		calldomain,
		numbersystem,
		calluri_name,
		user_id,
		user_fio,
		user_photo_name
		from FM_DETAIL_REPORT(:OF_ID,:STARTTM,:ENDTM) where 1=1 $employee_id $txt_calltype $txt_ats_number $txt_abonent_number;
EOS;
//		from FM_DETAIL_REPORT(147,'01.03.2017','01.05.2017');
		$q = $this->prepare($q);
		$q->bindValue(":OF_ID",$ofid);
		$q->bindValue(":STARTTM",$txt_date1);
		$q->bindValue(":ENDTM",$txt_date2);
		$q->execute();
		$result = $q->fetchAll();
		
		foreach($result as $key => $row){
			$row['creattime'] = date("d-m-Y H:i:s", strtotime($row['creattime']));
			$param = implode("|",$row);
			$tabrow[$key] = array(
				'sessionid' => $row['sessionid'],
				'creattime' => $row['creattime'],
				'numbera' => $row['numbera'],
				'numberb' => $row['numberb'],
				'realvolume' => $row['realvolume'],
				'geographyname' => $row['geographyname'],
				'distancename' => $row['distancename'],
				'pay' => $row['pay'],
				'recordid' => $row['recordid'],
				'termreason' => $row['termreason'],
				'cause' => $row['cause'],
				'answerdelay' => $row['answerdelay'],
				'flagsout' => $row['flagsout'],
				'calltype' => $row['calltype'],
				'payablevolume' => $row['payablevolume'],
				'serviceblockid' => $row['serviceblockid'],
				'messagetype' => $row['messagetype'],
				'floatcash1' => $row['floatcash1'],
				'floatcash2' => $row['floatcash2'],
				'floatgraph' => $row['floatgraph'],
				'calluri' => $row['calluri'],
				'calldomain' => $row['calldomain'],
				'numbersystem' => $row['numbersystem'],
				'calluri_name' => $row['calluri_name'],
				'user_id' => $row['user_id'],
				'user_fio' => $row['user_fio'],
				'user_photo_name' => $row['user_photo_name'],
				//'user_photo_file' => $row['user_photo_file'],
				//'count' => $q->rowCount(),
				'count' => $rowcount,
				'param' => $param
			);
		}
		return $tabrow;
	}

	
	
	
	
	
};

$database = new MYDB;

?>