<?php
require_once(dirname(__FILE__).'/constants.php');
require_once(dirname(__FILE__).'/database.php');
require_once(dirname(__FILE__).'/form.php');

class Session {
	
	public $userwebid;			// уникальный идентификатор пользователя (генератор случайных символов)
	public $syssession;			// переменные сессии (ассоциативный массив) &$_SESSION

/* свойства объекта класса создаются автоматически в момент присваивания значения (посредством $syssession[])
	$db_ok;				// признак наличия подключения к базе
	$user_id;			// идентификатор пользователя
	$session_name;		// имя сессии
	$session_id;		// идентификатор сессии
	$login_ok;			// признак успешной авторизации
	$start_time;		// время старта сессии
*/	
	
	//
	public function __set($name, $value) {
		$this->syssession[$this->userwebid][$name] = $value;
	}
	public function __get($name) {
		//if(isset($this->userwebid))
		if (array_key_exists($name, (array)$this->syssession[$this->userwebid]))
			return $this->syssession[$this->userwebid][$name];
		return null;
	}
	public function __isset($name) {
		return isset($this->syssession[$this->userwebid][$name]);
	}
	public function __unset($name) {
		unset($this->syssession[$this->userwebid][$name]);
	}

	// старт сессии
	function __construct() {
		$this->startSession();
	}
	
	function startSession() {
		global $database, $logger, $form;

		session_name('fmsession');
		session_start();

		$this->syssession = &$_SESSION;
		
		$this->start_time = time();
		$this->session_name = session_name();
		$this->session_id = session_id();
		

		// определение страницы перехода $session->referrer
		if (isset($this->url)) {
			if (preg_match('/(fs_(get|put)file.php)|(faxview.php)|(process.php)|(menu_(conditionprocessors|connprops|graph|itemnames|itemprops|itemtypes|officeprops|synthz).php)$/', $this->url))
				$this->referrer = isset($this->ref) ? $this->ref : '/';
			else
				$this->referrer = $this->ref = $this->url;
		} else
			$this->referrer = $this->ref = '../';
		//$logger->log('session.ref:'.$this->referrer);
		$this->url = $_SERVER['PHP_SELF'];


		//if($this->registration_ok) {$this->registration_ok = false;}
		//echo $_COOKIE['fmsession'].'<br><br>';
		//echo $this->session_id.'<br><br>';
		
	}

	
	function login($txt_login, $txt_password, $txt_remember) {
		global $database, $form, $logger;
		$txt_login = stripslashes($txt_login);
		$txt_password = stripslashes($txt_password);

		// наличие соединения с базой данных
		$this->db_ok = ($database->db_ok) ? true : false;
		$field = "login_db";
		if(!$this->db_ok) {
			$form->setError($field, "* database connection failed");
		} else {
			$logger->log("database: connected");
		}

		// проверка корректности имени пользователя
		$field = "login_log";
		if(!$txt_login || strlen($txt_login = trim($txt_login)) == 0) {
			$form->setError($field, "* user name is empty");
		}

		// проверка наличия пароля
		$field = "login_pass";
		if (!$txt_password){
			$form->setError($field, "* user password is empty");
		}

		// Return if form errors exist 
		if ($form->num_errors > 0) {
			$logger->log("login_form: error count = " . $form->num_errors);
			$form_error = $form->getErrorArray();
			foreach ($form_error as $key => $value) {
				if( strpos($key,"login_") !== false ) $logger->log($key.": ".$value);
			}
			return false;
		}

		$this->user_id = $database->confirmUserPass($txt_login, $txt_password);
		if ($this->user_id == -1) {
			$field = "login_user";
			$form->setError($field, "* login failed / log = ".$txt_login." pas = ".$txt_password);
			//$logger->log("user login: * failed / login = ".$txt_login." password = ".$txt_password);
			$logger->log($field.": ".$form->errors['login_user']);
		} else {
			$logger->log("user login: success access / user_id = ".$this->user_id);
		}

		// Return if form errors exist
		if ($form->num_errors > 0)
			return false;

		// доступ запрещен
		//if (!($userinfo = $database->getUserInfo($txt_login, $txt_password)))
		//return false;
		
		// доступ разрешен
		$this->logged_in = true;
		
		//$this->of_id = (int)$userinfo[0]['of_id'];
		//$this->user_id = (int)$userinfo[0]['user_id'];
		//$this->user_name = $txt_login;
		//$this->user_role = $userinfo[0]['user_role'];
		//$this->user_fio = $userinfo[0]['user_fio'];

		//$database->registerLoginSession($this->of_id,$this->user_id,$this->session_id,$this->start_time);
		/*
		if ($subremember) {
			setcookie(COOKIE_NAME_LOGINNAME, $this->username,  time()+COOKIE_EXPIRE, COOKIE_PATH);
			setcookie(COOKIE_NAME_LOGINPASS, $this->userwebid, time()+COOKIE_EXPIRE, COOKIE_PATH);
		}*/
		
		return $this->logged_in;
	}


	// выход из системы
	// файл сессии остается в session.save_path до удаления его сборщиком мусора, но параметры авторизации из него удаляются
	// при невозможности сопоставления куки клиента и сессии сервера 
	// например, удалили куку в браузере или файл сессии в session.save_path
	// будет создана новая пара - файл сессии и кука браузера
	// название файла = идентификатор сессии, содержимое куки = идентификатор сессии
	// в обоих случаях: осуществлен выход, или удалили куку или файл сессии, необходима повторная авторизация
	function logout(){
		global $database, $form, $logger;
		session_name('fmsession');
		session_start();
		session_unset();
		$_SESSION = array();
		setcookie(session_name(), session_id(), time()-60*60*24);
		session_destroy();		
	}


	/**
	 * generateRandID - Generates a string made up of randomized 
	 * letters (lower and upper case) and digits and returns
	 * the sha1 hash of it to be used as a userwebid.
	 * @param int $len
	 * @return string
	 */
	function generateRandID($len = 128) {
		return sha1($this->generateRandStr($len));
	}
	/**
	 * generateRandStr - Generates a string made up of randomized
	 * letters (lower and upper case) and digits, the length
	 * is a specified parameter.
	 * @param $length
	 * @return string
	 */
	function generateRandStr($length) {
		$randstr = "";
		for ($i=0; $i<$length; $i++) {
			$randnum = mt_rand(0,61);
			if($randnum < 10){
				$randstr .= chr($randnum+48);
			} else if ($randnum < 36) {
				$randstr .= chr($randnum+55);
			} else{
				$randstr .= chr($randnum+61);
			}
		}
		return $randstr;
	}



	
};

$session = new Session;
$form = new Form;

?>